from ...helpers import format_result
from .queries import GQL_DELETE_LOCKS


def delete_locks(client, asset_id: str):
    variables = {'assetID': asset_id}
    result = client.execute(GQL_DELETE_LOCKS, variables)
    return format_result('data', result)



from ...helpers import format_result
from .queries import (GQL_CREATE_USER,
                      GQL_CREATE_USER_FROM_EMAIL_IF_NOT_EXISTS,
                      GQL_RESET_PASSWORD, GQL_SIGN_IN, GQL_UPDATE_PASSWORD,
                      GQL_UPDATE_PROPERTIES_IN_USER, GQL_UPDATE_USER)


def signin(client, email: str, password: str):
    variables = {'email': email, 'password': password}
    result = client.execute(GQL_SIGN_IN, variables)
    return format_result('data', result)


def create_user(client, name: str, email: str, password: str, phone: str, organization_role: str):
    variables = {
        'name': name,
        'email': email,
        'password': password,
        'phone': phone,
        'organizationRole': organization_role
    }
    result = client.execute(GQL_CREATE_USER, variables)
    return format_result('data', result)


def create_user_from_email_if_not_exists(client, name: str, email: str, organization_role: str, project_id: str):
    variables = {
        'name': name,
        'email': email,
        'organizationRole': organization_role,
        'projectID': project_id
    }
    result = client.execute(
        GQL_CREATE_USER_FROM_EMAIL_IF_NOT_EXISTS, variables)
    return format_result('data', result)


def update_user(client, user_id: str, name: str, email: str, phone: str, organization_role: str):
    variables = {
        'userID': user_id,
        'name': name,
        'email': email,
        'phone': phone,
        'organizationRole': organization_role
    }
    result = client.execute(GQL_UPDATE_USER, variables)
    return format_result('data', result)


def update_password(client, email: str, old_password: str, new_password_1: str, new_password_2: str):
    variables = {
        'email': email,
        'oldPassword': old_password,
        'newPassword1': new_password_1,
        'newPassword2': new_password_2
    }
    result = client.execute(GQL_UPDATE_PASSWORD, variables)
    return format_result('data', result)


def reset_password(client, email: str):
    variables = {'email': email}
    result = client.execute(GQL_RESET_PASSWORD, variables)
    return format_result('data', result)


def update_properties_in_user(client, email: str, name: str = None, organization_id: str = None, organization_role: str = None, activated: bool = None):
    variables = {
        'email': email,
        'name': name,
        'organizationId': organization_id,
        'organizationRole': organization_role,
        'activated': activated
    }
    result = client.execute(GQL_UPDATE_PROPERTIES_IN_USER, variables)
    return format_result('data', result)



from json import dumps

from ...helpers import format_result
from .queries import (GQL_APPEND_TO_TOOLS, GQL_DELETE_FROM_TOOLS,
                      GQL_UPDATE_TOOL)


def update_tool(client, tool_id: str, project_id: str, json_settings: dict):
    variables = {
        'toolID': tool_id,
        'projectID': project_id,
        'jsonSettings': dumps(json_settings)
    }
    result = client.execute(GQL_UPDATE_TOOL, variables)
    return format_result('data', result)


def append_to_tools(client, project_id: str,  json_settings: dict):
    variables = {
        'projectID': project_id,
        'jsonSettings': dumps(json_settings)
    }
    result = client.execute(GQL_APPEND_TO_TOOLS, variables)
    return format_result('data', result)


def delete_from_tools(client, tool_id: str):
    variables = {'toolID': tool_id}
    result = client.execute(GQL_DELETE_FROM_TOOLS, variables)
    return format_result('data', result)



from .fragments import TOOL_FRAGMENT

GQL_UPDATE_TOOL = f'''
mutation($toolID: ID!, $projectID: ID!, $jsonSettings: String!) {{
  data: updateTool(toolID: $toolID
    projectID: $projectID
    jsonSettings: $jsonSettings) {{
      {TOOL_FRAGMENT}
  }}
}}
'''

GQL_APPEND_TO_TOOLS = f'''
mutation($projectID: ID!, $jsonSettings: String!) {{
  data: appendToTools(
    projectID: $projectID
    jsonSettings: $jsonSettings) {{
      {TOOL_FRAGMENT}
  }}
}}
'''

GQL_DELETE_FROM_TOOLS = f'''
mutation($toolID: ID!) {{
  data: deleteFromTools(toolID: $toolID) {{
    {TOOL_FRAGMENT}
  }}
}}
'''



import time
from json import dumps, loads
from typing import List

from tqdm import tqdm

from ...helpers import GraphQLError, format_result
from ...queries.asset import get_assets
from ...queries.label import get_label
from ...queries.project import get_project
from ..asset import update_properties_in_asset
from ..lock import delete_locks
from .queries import (GQL_APPEND_TO_ROLES, GQL_CREATE_EMPTY_PROJECT,
                      GQL_CREATE_PROJECT, GQL_DELETE_FROM_ROLES,
                      GQL_GQL_UPDATE_PROPERTIES_IN_PROJECT_USER,
                      GQL_UPDATE_PROJECT, GQL_UPDATE_PROPERTIES_IN_PROJECT,
                      GQL_UPDATE_ROLE)


def create_project(client, title: str, description: str, use_honeypot: bool,
                   interface_json_settings: dict):
    variables = {
        'title': title,
        'description': description,
        'useHoneyPot': use_honeypot,
        'interfaceJsonSettings': dumps(interface_json_settings)
    }
    result = client.execute(GQL_CREATE_PROJECT, variables)
    return format_result('data', result)


def append_to_roles(client, project_id: str, user_email: str, role: str):
    variables = {
        'projectID': project_id,
        'userEmail': user_email,
        'role': role
    }
    result = client.execute(GQL_APPEND_TO_ROLES, variables)
    return format_result('data', result)


def update_properties_in_project(client, project_id: str,
                                 title: str = None,
                                 description: str = None,
                                 min_consensus_size: int = None,
                                 consensus_tot_coverage: int = None,
                                 number_of_assets: int = None,
                                 number_of_remaining_assets: int = None,
                                 number_of_assets_with_empty_labels: int = None,
                                 number_of_reviewed_assets: int = None,
                                 number_of_latest_labels: int = None,
                                 consensus_mark: float = None,
                                 honeypot_mark: float = None,
                                 instructions: str = None):
    variables = {
        'projectID': project_id,
        'title': title,
        'description': description,
        'minConsensusSize': min_consensus_size,
        'consensusTotCoverage': consensus_tot_coverage,
        'numberOfAssets': number_of_assets,
        'numberOfRemainingAssets': number_of_remaining_assets,
        'numberOfAssetsWithSkippedLabels': number_of_assets_with_empty_labels,
        'numberOfReviewedAssets': number_of_reviewed_assets,
        'numberOfLatestLabels': number_of_latest_labels,
        'consensusMark': consensus_mark,
        'honeypotMark': honeypot_mark,
        'instructions': instructions
    }
    result = client.execute(GQL_UPDATE_PROPERTIES_IN_PROJECT, variables)
    return format_result('data', result)


def create_empty_project(client, user_id: str):
    variables = {'userID': user_id}
    result = client.execute(GQL_CREATE_EMPTY_PROJECT, variables)
    return format_result('data', result)


def update_project(client, project_id: str,
                   title: str,
                   description: str,
                   interface_category: str,
                   input_type: str = 'TEXT',
                   consensus_tot_coverage: int = 0,
                   min_consensus_size: int = 1,
                   max_worker_count: int = 4,
                   min_agreement: int = 66,
                   use_honey_pot: bool = False,
                   instructions: str = None):
    variables = {
        'projectID': project_id,
        'title': title,
        'description': description,
        'interfaceCategory': interface_category,
        'inputType': input_type,
        'consensusTotCoverage': consensus_tot_coverage,
        'minConsensusSize': min_consensus_size,
        'maxWorkerCount': max_worker_count,
        'minAgreement': min_agreement,
        'useHoneyPot': use_honey_pot,
        'instructions': instructions
    }
    result = client.execute(GQL_UPDATE_PROJECT, variables)
    return format_result('data', result)


def update_role(client, role_id: str, project_id: str, user_id: str, role: str):
    variables = {
        'roleID': role_id,
        'projectID': project_id,
        'userID': user_id,
        'role': role
    }
    result = client.execute(GQL_UPDATE_ROLE, variables)
    return format_result('data', result)


def delete_from_roles(client, role_id: str):
    variables = {'roleID': role_id}
    result = client.execute(GQL_DELETE_FROM_ROLES, variables)
    return format_result('data', result)


def update_properties_in_project_user(client, project_user_id: str,
                                      total_duration: int = None,
                                      number_of_labeled_assets: int = None,
                                      consensus_mark: float = None,
                                      honeypot_mark: float = None):
    variables = {
        'projectUserID': project_user_id,
        'totalDuration': total_duration,
        'numberOfLabeledAssets': number_of_labeled_assets,
        'consensusMark': consensus_mark,
        'honeypotMark': honeypot_mark
    }
    result = client.execute(
        GQL_GQL_UPDATE_PROPERTIES_IN_PROJECT_USER, variables)
    return format_result('data', result)


def force_project_kpis(client, project_id: str):
    _ = get_assets(client, project_id=project_id)
    _ = get_project(client, project_id=project_id)



from ...helpers import format_result
from .queries import GQL_GET_PROJECT, GQL_GET_PROJECTS


def get_projects(client, user_id: str, search_query=None, skip=0, first=100):
    variables = {'userID': user_id, 'searchQuery': search_query, 'skip': skip, 'first': first}
    result = client.execute(GQL_GET_PROJECTS, variables)
    return format_result('data', result)


def get_project(client, project_id: str):
    variables = {'projectID': project_id}
    result = client.execute(GQL_GET_PROJECT, variables)
    return format_result('data', result)



from .fragments import ASSET_FRAGMENT

GQL_DELETE_ASSETS_BY_EXTERNAL_ID = f'''
mutation($projectID: ID!, $externalID: String!) {{
  data: deleteAssetsByExternalId(projectID: $projectID, externalID: $externalID) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_APPEND_MANY_TO_DATASET = f'''
mutation(
    $projectID: ID!,
    $contentArray: [String!],
    $externalIDArray: [String!],
    $isHoneypotArray: [Boolean!],
    $statusArray: [Status!],
    $jsonMetadataArray: [String!]) {{
  data: appendManyToDataset(
    projectID: $projectID,
    contentArray: $contentArray,
    externalIDArray: $externalIDArray,
    isHoneypotArray: $isHoneypotArray,
    statusArray: $statusArray,
    jsonMetadataArray: $jsonMetadataArray
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_ASSET = f'''
mutation(
    $assetID: ID!,
    $projectID: ID!,
    $content: String!,
    $externalID: String,
    $isHoneypot: Boolean,
    $consensusMark: Float,
    $honeypotMark: Float,
    $status:  Status!,
    $jsonMetadata: String
) {{
  data: updateAsset(
    assetID: $assetID,
    projectID: $projectID,
    content: $content,
    externalID: $externalID,
    isHoneypot: $isHoneypot,
    consensusMark: $consensusMark,
    honeypotMark: $honeypotMark,
    status: $status,
    jsonMetadata: $jsonMetadata) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_PROPERTIES_IN_ASSET = f'''
mutation(
    $assetID: ID!
    $externalId: String
    $priority: Int
    $jsonMetadata: String
    $consensusMark: Float
    $honeypotMark: Float
    $toBeLabeledBy: [String!]
) {{
  data: updatePropertiesInAsset(
    where: {{id: $assetID}},
    data: {{
      externalId: $externalId
      priority: $priority
      jsonMetadata: $jsonMetadata
      consensusMark: $consensusMark
      honeypotMark: $honeypotMark
      toBeLabeledBy: $toBeLabeledBy
    }}
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_DELETE_FROM_DATASET = f'''
mutation($assetID: ID!) {{
  data: deleteFromDataset(assetID: $assetID) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_DELETE_MANY_FROM_DATASET = f'''
mutation($assetIDs: [ID!]) {{
  data: deleteManyFromDataset(assetIDs: $assetIDs) {{
    {ASSET_FRAGMENT}
  }}
}}
'''



from .fragments import PROJECT_FRAGMENT, PROJECT_FRAGMENT_ID

GQL_CREATE_PROJECT = f'''
mutation(
    $title: String!
    $description: String!
    $useHoneyPot: Boolean!
    $interfaceJsonSettings: String!
) {{
  data: createProject(
  title: $title
  description: $description
  useHoneyPot: $useHoneyPot
  interfaceJsonSettings: $interfaceJsonSettings) {{
    {PROJECT_FRAGMENT_ID}
  }}
}}
'''

GQL_APPEND_TO_ROLES = f'''
mutation($projectID: ID!, $userEmail: String!, $role: ProjectRole!) {{
  data: appendToRoles(
    projectID: $projectID
    userEmail: $userEmail
    role: $role) {{
      {PROJECT_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_PROPERTIES_IN_PROJECT = f'''
mutation(
    $projectID: ID!
    $title: String
    $description: String
    $minConsensusSize: Int
    $consensusTotCoverage: Int
    $numberOfAssets: Int
    $numberOfRemainingAssets: Int
    $numberOfAssetsWithSkippedLabels: Int
    $numberOfReviewedAssets: Int
    $numberOfLatestLabels: Int
    $consensusMark: Float
    $honeypotMark: Float
    $instructions: String
    $jsonSettings: String
) {{
  data: updatePropertiesInProject(
    where: {{id: $projectID}},
    data: {{
      title: $title
      description: $description
      minConsensusSize: $minConsensusSize
      consensusTotCoverage: $consensusTotCoverage
      numberOfAssets: $numberOfAssets
      numberOfRemainingAssets: $numberOfRemainingAssets
      numberOfAssetsWithSkippedLabels: $numberOfAssetsWithSkippedLabels
      numberOfReviewedAssets: $numberOfReviewedAssets
      numberOfLatestLabels: $numberOfLatestLabels
      consensusMark: $consensusMark
      honeypotMark: $honeypotMark
      instructions: $instructions
      jsonSettings: $jsonSettings
    }}
  ) {{
    {PROJECT_FRAGMENT_ID}
  }}
}}
'''


GQL_CREATE_EMPTY_PROJECT = f'''
mutation($userID: ID!) {{
  data: createEmptyProject(userID: $userID) {{
    {PROJECT_FRAGMENT_ID}
  }}
}}
'''

GQL_UPDATE_PROJECT = f'''
mutation(
    $projectID: ID!
    $title: String!
    $description: String
    $interfaceCategory: InterfaceCategory
    $inputType: InputType
    $consensusTotCoverage: Int
    $minConsensusSize: Int
    $maxWorkerCount: Int
    $minAgreement: Int
    $useHoneyPot: Boolean
    $instructions: String
) {{
  data: updateProject(
    projectID: $projectID
    title: $title
    description: $description
    interfaceCategory: $interfaceCategory
    inputType: $inputType
    consensusTotCoverage: $consensusTotCoverage
    minConsensusSize: $minConsensusSize
    maxWorkerCount: $maxWorkerCount
    minAgreement: $minAgreement
    useHoneyPot: $useHoneyPot
    instructions: $instructions) {{
      {PROJECT_FRAGMENT_ID}
  }}
}}
'''

GQL_UPDATE_ROLE = f'''
mutation(
    $roleID: ID!
    $projectID: ID!
    $userID: ID!
    $role: ProjectRole!
) {{
  data: updateRole(
    roleID: $roleID
    projectID: $projectID
    userID: $userID
    role: $role) {{
      {PROJECT_FRAGMENT_ID}
  }}
}}
'''

GQL_DELETE_FROM_ROLES = f'''
mutation($roleID: ID!) {{
  data: deleteFromRoles(roleID: $roleID) {{
    {PROJECT_FRAGMENT_ID}
  }}
}}
'''

GQL_GQL_UPDATE_PROPERTIES_IN_PROJECT_USER = f'''
mutation(
    $projectID: ID!
    $totalDuration: Int
    $numberOfLabeledAssets: Int
    $consensusMark: Float
    $honeypotMark: Float
) {{
  data: updatePropertiesInProjectUser(
    where: {{id: $projectID}},
    data: {{
      totalDuration: $totalDuration
      numberOfLabeledAssets: $numberOfLabeledAssets
      consensusMark: $consensusMark
      honeypotMark: $honeypotMark
    }}
  ) {{
    {PROJECT_FRAGMENT_ID}
  }}
}}
'''



import kili.mutations.asset
import kili.mutations.label
import kili.mutations.lock
import kili.mutations.organization
import kili.mutations.project
import kili.mutations.tool
import kili.mutations.user
import kili.queries.asset
import kili.queries.label
import kili.queries.lock
import kili.queries.project
import kili.queries.tool
import kili.queries.user
import kili.subscriptions.label


class Playground(object):

    def __init__(self, auth=None):
        """Create an instance of KiliPlayground."""
        self.auth = auth

    # Mutations Asset

    def delete_assets_by_external_id(self, **kwargs):
        return kili.mutations.asset.delete_assets_by_external_id(self.auth.client, **kwargs)

    def append_many_to_dataset(self, **kwargs):
        return kili.mutations.asset.append_many_to_dataset(self.auth.client, **kwargs)

    def update_asset(self, **kwargs):
        return kili.mutations.asset.update_asset(self.auth.client, **kwargs)

    def update_properties_in_asset(self, **kwargs):
        return kili.mutations.asset.update_properties_in_asset(self.auth.client, **kwargs)

    def delete_from_dataset(self, **kwargs):
        return kili.mutations.asset.delete_from_dataset(self.auth.client, **kwargs)

    def delete_many_from_dataset(self, **kwargs):
        return kili.mutations.asset.delete_many_from_dataset(self.auth.client, **kwargs)

    # Mutations Label

    def create_prediction(self, **kwargs):
        return kili.mutations.label.create_prediction(self.auth.client, **kwargs)

    def create_predictions(self, **kwargs):
        return kili.mutations.label.create_predictions(self.auth.client, **kwargs)

    def append_to_labels(self, **kwargs):
        return kili.mutations.label.append_to_labels(self.auth.client, **kwargs)

    def update_label(self, **kwargs):
        return kili.mutations.label.update_label(self.auth.client, **kwargs)

    def update_properties_in_label(self, **kwargs):
        return kili.mutations.label.update_properties_in_label(self.auth.client, **kwargs)

    def create_honeypot(self, **kwargs):
        return kili.mutations.label.create_honeypot(self.auth.client, **kwargs)

    # Mutations Lock

    def delete_locks(self, **kwargs):
        return kili.mutations.lock.delete_locks(self.auth.client, **kwargs)

    # Mutations Organization

    def create_organization(self, **kwargs):
        return kili.mutations.organization.create_organization(self.auth.client, **kwargs)

    def update_organization(self, **kwargs):
        return kili.mutations.organization.update_organization(self.auth.client, **kwargs)

    def delete_organization(self, **kwargs):
        return kili.mutations.organization.delete_organization(self.auth.client, **kwargs)

    # Mutations Project

    def create_project(self, **kwargs):
        return kili.mutations.project.create_project(self.auth.client, **kwargs)

    def append_to_roles(self, **kwargs):
        return kili.mutations.project.append_to_roles(self.auth.client, **kwargs)

    def update_properties_in_project(self, **kwargs):
        return kili.mutations.project.update_properties_in_project(self.auth.client, **kwargs)

    def create_empty_project(self, **kwargs):
        return kili.mutations.project.create_empty_project(self.auth.client, **kwargs)

    def update_project(self, **kwargs):
        return kili.mutations.project.update_project(self.auth.client, **kwargs)

    def update_role(self, **kwargs):
        return kili.mutations.project.update_role(self.auth.client, **kwargs)

    def delete_from_roles(self, **kwargs):
        return kili.mutations.project.delete_from_roles(self.auth.client, **kwargs)

    def update_properties_in_project_user(self, **kwargs):
        return kili.mutations.project.update_properties_in_project_user(self.auth.client, **kwargs)

    def force_project_kpis(self, **kwargs):
        return kili.mutations.project.force_project_kpis(self.auth.client, **kwargs)

    # Mutations Tool

    def update_tool(self, **kwargs):
        return kili.mutations.tool.update_tool(self.auth.client, **kwargs)

    def append_to_tools(self, **kwargs):
        return kili.mutations.tool.append_to_tools(self.auth.client, **kwargs)

    def delete_from_tools(self, **kwargs):
        return kili.mutations.tool.delete_from_tools(self.auth.client, **kwargs)

    # Mutations User

    def create_user(self, **kwargs):
        return kili.mutations.user.create_user(self.auth.client, **kwargs)

    def create_user_from_email_if_not_exists(self, **kwargs):
        return kili.mutations.user.create_user_from_email_if_not_exists(self.auth.client, **kwargs)

    def update_user(self, **kwargs):
        return kili.mutations.user.update_user(self.auth.client, **kwargs)

    def update_password(self, **kwargs):
        return kili.mutations.user.update_password(self.auth.client, **kwargs)

    def reset_password(self, **kwargs):
        return kili.mutations.user.reset_password(self.auth.client, **kwargs)

    def update_properties_in_user(self, **kwargs):
        return kili.mutations.user.update_properties_in_user(self.auth.client, **kwargs)

    # Queries Asset

    def get_asset(self, **kwargs):
        return kili.queries.asset.get_asset(self.auth.client, **kwargs)

    def get_assets(self, **kwargs):
        return kili.queries.asset.get_assets(self.auth.client, **kwargs)

    def get_assets_with_search(self, **kwargs):
        return kili.queries.asset.get_assets_with_search(self.auth.client, **kwargs)

    def get_assets_by_external_id(self, **kwargs):
        return kili.queries.asset.get_assets_by_external_id(self.auth.client, **kwargs)

    def get_next_asset_from_label(self, **kwargs):
        return kili.queries.asset.get_next_asset_from_label(self.auth.client, **kwargs)

    def get_next_asset_from_project(self, **kwargs):
        return kili.queries.asset.get_next_asset_from_project(self.auth.client, **kwargs)

    # Queries Label

    def get_label(self, **kwargs):
        return kili.queries.label.get_label(self.auth.client, **kwargs)

    def get_latest_labels_for_user(self, **kwargs):
        return kili.queries.label.get_latest_labels_for_user(self.auth.client, **kwargs)

    def get_latest_labels(self, **kwargs):
        return kili.queries.label.get_latest_labels(self.auth.client, **kwargs)

    def export_labels_as_df(self, **kwargs):
        return kili.queries.label.export_labels_as_df(self.auth.client, **kwargs)

    # Queries Lock

    def get_locks(self, **kwargs):
        return kili.queries.lock.get_locks(self.auth.client, **kwargs)

    # Queries Project

    def get_projects(self, **kwargs):
        return kili.queries.project.get_projects(self.auth.client, **kwargs)

    def get_project(self, **kwargs):
        return kili.queries.project.get_project(self.auth.client, **kwargs)

    # Queries Tool

    def get_tools(self, **kwargs):
        return kili.queries.tool.get_tools(self.auth.client, **kwargs)

    # Queries User

    def get_user(self, **kwargs):
        return kili.queries.user.get_user(self.auth.client, **kwargs)

    # Subscriptions Label

    def label_created_or_updated(self, **kwargs):
        return kili.subscriptions.label.label_created_or_updated(self.auth.client, **kwargs)


if __name__ == '__main__':
    """ Example of usage """
    from kili.authentication import KiliAuth
    from kili.playground import Playground
    kauth = KiliAuth()
    playground = Playground(kauth)
    assets = playground.export_assets(project_id="first-project")
    print(assets)



from json import dumps
from typing import List

from ...helpers import format_result
from .queries import (GQL_APPEND_TO_LABELS, GQL_CREATE_HONEYPOT,
                      GQL_CREATE_PREDICTIONS, GQL_UPDATE_LABEL,
                      GQL_UPDATE_PROPERTIES_IN_LABEL)


def create_predictions(client, project_id: str, external_id_array: List[str], model_name_array: List[str], json_response_array: List[dict]):
    assert len(external_id_array) == len(
        json_response_array), "IDs list and predictions list should have the same length"
    assert len(external_id_array) == len(
        model_name_array), "IDs list and model names list should have the same length"
    variables = {
        'projectID': project_id,
        'externalIDArray': external_id_array,
        'modelNameArray': model_name_array,
        'jsonResponseArray': [dumps(elem) for elem in json_response_array]
    }
    result = client.execute(GQL_CREATE_PREDICTIONS, variables)
    return format_result('data', result)


def append_to_labels(client, author_id: str, json_response: dict, label_asset_id: str, label_type: str, seconds_to_label: int, skipped: bool = False):
    variables = {
        'authorID': author_id,
        'jsonResponse': dumps(json_response),
        'labelAssetID': label_asset_id,
        'labelType': label_type,
        'secondsToLabel': seconds_to_label,
        'skipped': skipped
    }
    result = client.execute(GQL_APPEND_TO_LABELS, variables)
    return format_result('data', result)


def update_label(client, label_id: str, label_asset_id: str, review_asset_id: str, author_id: str, label_type: str, json_response: dict, seconds_to_label: int):
    variables = {
        'labelID': label_id,
        'labelAssetID': label_asset_id,
        'reviewAssetID': review_asset_id,
        'authorID': author_id,
        'labelType': label_type,
        'jsonResponse': dumps(json_response),
        'secondsToLabel': seconds_to_label
    }
    result = client.execute(GQL_UPDATE_LABEL, variables)
    return format_result('data', result)


def update_properties_in_label(client, label_id: str, seconds_to_label: int = None, model_name: str = None, json_response: dict = None):
    formatted_json_response = None if json_response is None else dumps(
        json_response)
    variables = {
        'labelID': label_id,
        'secondsToLabel': seconds_to_label,
        'modelName': model_name,
        'jsonResponse': formatted_json_response
    }
    result = client.execute(GQL_UPDATE_PROPERTIES_IN_LABEL, variables)
    return format_result('data', result)


def create_honeypot(client, asset_id: str, json_response: dict):
    variables = {
        'assetID': asset_id,
        'jsonResponse': dumps(json_response)
    }
    result = client.execute(GQL_CREATE_HONEYPOT, variables)
    return format_result('data', result)



from typing import Any, Callable

from ...helpers import format_result
from .subscriptions import GQL_LABEL_CREATED_OR_UPDATED
from ...graphql_client import SubscriptionGraphQLClient


def label_created_or_updated(client, project_id: str, callback: Callable[[str, str], None]):
    ws_endpoint = client.endpoint.replace('http', 'ws')
    ws = SubscriptionGraphQLClient(ws_endpoint)
    headers = {'Accept': 'application/json',
               'Content-Type': 'application/json'}
    headers['Authorization'] = f'{client.token}'
    variables = {'projectID': project_id}
    ws.subscribe(GQL_LABEL_CREATED_OR_UPDATED, variables=variables,
                 callback=callback, headers=headers)



from json import dumps
from typing import List

from ...helpers import content_escape, encode_image, format_result, is_url
from ...queries.project import get_project
from .queries import (GQL_APPEND_MANY_TO_DATASET,
                      GQL_DELETE_ASSETS_BY_EXTERNAL_ID,
                      GQL_DELETE_FROM_DATASET, GQL_DELETE_MANY_FROM_DATASET,
                      GQL_UPDATE_ASSET, GQL_UPDATE_PROPERTIES_IN_ASSET)


def delete_assets_by_external_id(client, project_id: str, external_id: str):
    variables = {'projectID': project_id, 'externalID': external_id}
    result = client.execute(GQL_DELETE_ASSETS_BY_EXTERNAL_ID, variables)
    return format_result('data', result)


def append_many_to_dataset(client, project_id: str, content_array: List[str], external_id_array: List[str],
                           is_honeypot_array: List[bool] = None, status_array: List[str] = None, json_metadata_array: List[dict] = None):
    is_honeypot_array = [
        False] * len(content_array) if not is_honeypot_array else is_honeypot_array
    status_array = ['TODO'] * \
        len(content_array) if not status_array else status_array
    json_metadata_array = [
        {}] * len(content_array) if not json_metadata_array else json_metadata_array
    formatted_json_metadata_array = [
        dumps(elem) for elem in json_metadata_array]
    input_type = get_project(client, project_id)['inputType']
    if input_type == 'IMAGE':
        content_array = [content if is_url(content) else encode_image(
            content) for content in content_array]
    variables = {
        'projectID': project_id,
        'contentArray': content_array,
        'externalIDArray': external_id_array,
        'isHoneypotArray': is_honeypot_array,
        'statusArray': status_array,
        'jsonMetadataArray': formatted_json_metadata_array}
    result = client.execute(GQL_APPEND_MANY_TO_DATASET, variables)
    return format_result('data', result)


def update_asset(client, asset_id: str, project_id: str, content: str, external_id: str,
                 is_honeypot: bool, consensus_mark: float, honeypot_mark: float, status: str, json_metadata: dict):
    formatted_json_metadata = dumps(json_metadata)
    variables = {
        'assetID': asset_id,
        'projectID': project_id,
        'content': content,
        'externalID': external_id,
        'isHoneypot': is_honeypot,
        'consensusMark': consensus_mark,
        'honeypotMark': honeypot_mark,
        'status': status,
        'jsonMetadata': formatted_json_metadata}
    result = client.execute(GQL_UPDATE_ASSET, variables)
    return format_result('data', result)


def update_properties_in_asset(client, asset_id: str, external_id: str = None,
                               priority: int = None, json_metadata: dict = None, consensus_mark: float = None,
                               honeypot_mark: float = None, to_be_labeled_by: List[str] = None):
    formatted_json_metadata = None
    if json_metadata is None:
        formatted_json_metadata = None
    elif isinstance(json_metadata, str):
        formatted_json_metadata = json_metadata
    elif isinstance(json_metadata, dict) or isinstance(json_metadata, list):
        formatted_json_metadata = dumps(json_metadata)
    else:
        raise Exception('json_metadata',
                        'Should be either a dict, a list or a string url')
    variables = {
        'assetID': asset_id,
        'externalID': external_id,
        'priority': priority,
        'jsonMetadata': formatted_json_metadata,
        'consensusMark': consensus_mark,
        'honeypotMark': honeypot_mark,
        'toBeLabeledBy': to_be_labeled_by
    }
    result = client.execute(GQL_UPDATE_PROPERTIES_IN_ASSET, variables)
    return format_result('data', result)


def delete_from_dataset(client, asset_id: str):
    variables = {'assetID': asset_id}
    result = client.execute(GQL_DELETE_FROM_DATASET, variables)
    return format_result('data', result)


def delete_many_from_dataset(client, asset_ids: List[str]):
    variables = {'assetIDs': asset_ids}
    result = client.execute(GQL_DELETE_MANY_FROM_DATASET, variables)
    return format_result('data', result)



import time
from json import dumps
from typing import List

import pandas as pd
from tqdm import tqdm

from ...helpers import format_result
from .queries import (GQL_COUNT_ASSETS_WITH_SEARCH, GQL_EXPORT_ASSETS,
                      GQL_GET_ASSET, GQL_GET_ASSETS_BY_EXTERNAL_ID,
                      GQL_GET_ASSETS_WITH_SEARCH,
                      GQL_GET_NEXT_ASSET_FROM_LABEL,
                      GQL_GET_NEXT_ASSET_FROM_PROJECT)


def get_asset(client, asset_id: str):
    variables = {'assetID': asset_id}
    result = client.execute(GQL_GET_ASSET, variables)
    return format_result('data', result)


def get_assets(client, project_id: str, skip: int = 0, first: int = None,
               external_id_contains: List[str] = None,
               status_in: List[str] = None,
               author_in: List[str] = None,
               consensus_mark_gt: float = None,
               consensus_mark_lt: float = None,
               honeypot_mark_gt: float = None,
               honeypot_mark_lt: float = None,
               skipped: bool = None,
               label_external_id_contains: str = None,
               label_type_in: List[str] = None,
               label_status_in: List[str] = None,
               label_author_in: List[str] = None,
               label_consensus_mark_gt: float = None,
               label_consensus_mark_lt: float = None,
               label_honeypot_mark_gt: float = None,
               label_honeypot_mark_lt: float = None,
               label_created_at_gt: float = None,
               label_created_at_lt: float = None,
               label_skipped: bool = None,
               format: str = None, disable_tqdm: bool = False):
    saved_args = locals()
    count_args = {k: v for (k, v) in saved_args.items()
                  if k not in ['skip', 'first', 'disable_tqdm', 'format']}
    number_of_assets_with_search = count_assets(**count_args)
    total = min(number_of_assets_with_search,
                first) if first is not None else number_of_assets_with_search
    formatted_first = first if first else 100
    if total == 0:
        return []
    with tqdm(total=total, disable=disable_tqdm) as pbar:
        paged_assets = []
        while True:
            variables = {
                'projectID': project_id,
                'skip': skip,
                'first': formatted_first,
                'externalIdIn': external_id_contains,
                'statusIn': status_in,
                'authorIn': author_in,
                'consensusMarkGte': consensus_mark_gt,
                'consensusMarkLte': consensus_mark_lt,
                'honeypotMarkGte': honeypot_mark_gt,
                'honeypotMarkLte': honeypot_mark_lt,
                'skipped': skipped,
                'labelExternalIdContains': label_external_id_contains,
                'labelTypeIn': label_type_in,
                'labelStatusIn': label_status_in,
                'labelAuthorIn': label_author_in,
                'labelConsensusMarkGte': label_consensus_mark_gt,
                'labelConsensusMarkLte': label_consensus_mark_lt,
                'labelHoneypotMarkGte': label_honeypot_mark_gt,
                'labelHoneypotMarkLte': label_honeypot_mark_lt,
                'labelCreatedAtGte': label_created_at_gt,
                'labelCreatedAtLte': label_created_at_lt,
                'labelSkipped': label_skipped,
            }
            result = client.execute(GQL_GET_ASSETS_WITH_SEARCH, variables)
            assets = format_result('data', result)
            if assets is None or len(assets) == 0 or (first is not None and len(paged_assets) == first):
                if format == 'pandas':
                    return pd.DataFrame(paged_assets)
                return paged_assets
            if first is not None:
                assets = assets[:max(0, first - len(paged_assets))]
            paged_assets += assets
            skip += formatted_first
            pbar.update(len(assets))


def get_assets_by_external_id(client, project_id: str, external_id: str):
    variables = {'projectID': project_id, 'externalID': external_id}
    result = client.execute(GQL_GET_ASSETS_BY_EXTERNAL_ID, variables)
    return format_result('data', result)


def get_next_asset_from_label(client, label_asset_ids: List[str]):
    variables = {'labelAssetIDs': label_asset_ids}
    result = client.execute(GQL_GET_NEXT_ASSET_FROM_LABEL, variables)
    return format_result('data', result)


def get_next_asset_from_project(client, project_id: str):
    variables = {'projectID': project_id}
    result = client.execute(GQL_GET_NEXT_ASSET_FROM_PROJECT, variables)
    return format_result('data', result)


def export_assets(client, project_id: str):
    variables = {'projectID': project_id}
    result = client.execute(GQL_EXPORT_ASSETS, variables)
    return format_result('exportAssets', result)


def count_assets(client, project_id: str,
                 external_id_contains: List[str] = None,
                 status_in: List[str] = None,
                 author_in: List[str] = None,
                 consensus_mark_gt: float = None,
                 consensus_mark_lt: float = None,
                 honeypot_mark_gt: float = None,
                 honeypot_mark_lt: float = None,
                 skipped: bool = None,
                 label_external_id_contains: str = None,
                 label_type_in: List[str] = None,
                 label_status_in: List[str] = None,
                 label_author_in: List[str] = None,
                 label_consensus_mark_gt: float = None,
                 label_consensus_mark_lt: float = None,
                 label_honeypot_mark_gt: float = None,
                 label_honeypot_mark_lt: float = None,
                 label_created_at_gt: float = None,
                 label_created_at_lt: float = None,
                 label_skipped: bool = None):
    variables = {
        'projectID': project_id,
        'externalIdIn': external_id_contains,
        'statusIn': status_in,
        'authorIn': author_in,
        'consensusMarkGte': consensus_mark_gt,
        'consensusMarkLte': consensus_mark_lt,
        'honeypotMarkGte': honeypot_mark_gt,
        'honeypotMarkLte': honeypot_mark_lt,
        'skipped': skipped,
        'labelExternalIdContains': label_external_id_contains,
        'labelTypeIn': label_type_in,
        'labelStatusIn': label_status_in,
        'labelAuthorIn': label_author_in,
        'labelConsensusMarkGte': label_consensus_mark_gt,
        'labelConsensusMarkLte': label_consensus_mark_lt,
        'labelHoneypotMarkGte': label_honeypot_mark_gt,
        'labelHoneypotMarkLte': label_honeypot_mark_lt,
        'labelCreatedAtGte': label_created_at_gt,
        'labelCreatedAtLte': label_created_at_lt,
        'labelSkipped': label_skipped,
    }
    result = client.execute(GQL_COUNT_ASSETS_WITH_SEARCH, variables)
    count = format_result('data', result)
    return count



from .fragments import LABEL_FRAGMENT

GQL_GET_LABEL = f'''
query ($assetID: ID!, $userID: ID!) {{
  data: getLabel(assetID: $assetID, userID: $userID) {{
    {LABEL_FRAGMENT}
  }}
}}
'''

GQL_GET_LATEST_LABELS_FOR_USER = f'''
query($projectID: ID!, $userID: ID!) {{
  data: getLatestLabelsForUser(projectID: $projectID, userID: $userID) {{
    {LABEL_FRAGMENT}
  }}
}}
'''

GQL_GET_LATEST_LABELS = f'''
query($projectID: ID!, $skip: Int!, $first: Int!) {{
  data: getLatestLabels(projectID: $projectID, skip: $skip, first: $first) {{
    {LABEL_FRAGMENT}
  }}
}}
'''



from ...helpers import format_result
from .queries import GQL_GET_TOOLS


def get_tools(client, project_id: str):
    variables = {'projectID': project_id}
    result = client.execute(GQL_GET_TOOLS, variables)
    return format_result('data', result)



from .fragments import ASSET_FRAGMENT, ASSET_FRAGMENT_SIMPLIFIED

GQL_GET_ASSET = f'''
query($assetID: ID!) {{
  data: getAsset(assetID: $assetID) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_GET_ASSETS_WITH_SEARCH = f'''
query(
    $projectID: ID!
    $skip: Int!
    $first: Int!
    $externalIdIn: [String]
    $statusIn: [String]
    $authorIn: [String]
    $consensusMarkGte: Float
    $consensusMarkLte: Float
    $honeypotMarkGte: Float
    $honeypotMarkLte: Float
    $skipped: Boolean
    $labelExternalIdContains: String
    $labelTypeIn: [String]
    $labelStatusIn: [String]
    $labelAuthorIn: [String]
    $labelConsensusMarkGte: Float
    $labelConsensusMarkLte: Float
    $labelHoneypotMarkGte: Float
    $labelHoneypotMarkLte: Float
    $labelCreatedAtGte: String
    $labelCreatedAtLte: String
    $labelSkipped: Boolean

) {{
  data: getAssetsWithSearch(
    projectID: $projectID
    skip: $skip
    first: $first
    assetsWhere: {{
      externalIdIn: $externalIdIn
      statusIn: $statusIn
      authorIn: $authorIn
      consensusMarkGte: $consensusMarkGte
      consensusMarkLte: $consensusMarkLte
      honeypotMarkGte: $honeypotMarkGte
      honeypotMarkLte: $honeypotMarkLte
      skipped: $skipped
    }}
    labelsWhere: {{
      externalIdContains: $labelExternalIdContains
      typeIn: $labelTypeIn
      statusIn: $labelStatusIn
      authorIn: $labelAuthorIn
      consensusMarkGte: $labelConsensusMarkGte
      consensusMarkLte: $labelConsensusMarkLte
      honeypotMarkGte: $labelHoneypotMarkGte
      honeypotMarkLte: $labelHoneypotMarkLte
      createdAtGte: $labelCreatedAtGte
      createdAtLte: $labelCreatedAtLte
      skipped: $labelSkipped
    }}
  ) {{
    {ASSET_FRAGMENT_SIMPLIFIED}
  }}
}}
'''

GQL_GET_ASSETS_BY_EXTERNAL_ID = f'''
query($projectID: ID!, $externalID: String!) {{
  data: getAssetsByExternalId(projectID: $projectID, externalID: $externalID) {{
    {ASSET_FRAGMENT_SIMPLIFIED}
  }}
}}
'''

GQL_GET_NEXT_ASSET_FROM_LABEL = f'''
query($labelAssetIDs: [ID!]) {{
  data: getNextAssetFromLabel(labelAssetIDs: $labelAssetIDs, where: {{}}) {{
    {ASSET_FRAGMENT_SIMPLIFIED}
  }}
}}
'''

GQL_GET_NEXT_ASSET_FROM_PROJECT = f'''
query($projectID: ID!) {{
  data: getNextAssetFromProject(projectID: $projectID) {{
    {ASSET_FRAGMENT_SIMPLIFIED}
  }}
}}
'''

GQL_EXPORT_ASSETS = f'''
query($projectID: ID!) {{
  exportAssets(projectID: $projectID) {{
    {ASSET_FRAGMENT_SIMPLIFIED}
  }}
}}
'''

GQL_COUNT_ASSETS_WITH_SEARCH = f'''
query(
    $projectID: ID!
    $externalIdIn: [String]
    $statusIn: [String]
    $authorIn: [String]
    $consensusMarkGte: Float
    $consensusMarkLte: Float
    $honeypotMarkGte: Float
    $honeypotMarkLte: Float
    $skipped: Boolean
    $labelExternalIdContains: String
    $labelTypeIn: [String]
    $labelStatusIn: [String]
    $labelAuthorIn: [String]
    $labelConsensusMarkGte: Float
    $labelConsensusMarkLte: Float
    $labelHoneypotMarkGte: Float
    $labelHoneypotMarkLte: Float
    $labelCreatedAtGte: String
    $labelCreatedAtLte: String
    $labelSkipped: Boolean

) {{
  data: countAssetsWithSearch(
    projectID: $projectID
    assetsWhere: {{
      externalIdIn: $externalIdIn
      statusIn: $statusIn
      authorIn: $authorIn
      consensusMarkGte: $consensusMarkGte
      consensusMarkLte: $consensusMarkLte
      honeypotMarkGte: $honeypotMarkGte
      honeypotMarkLte: $honeypotMarkLte
      skipped: $skipped
    }}
    labelsWhere: {{
      externalIdContains: $labelExternalIdContains
      typeIn: $labelTypeIn
      statusIn: $labelStatusIn
      authorIn: $labelAuthorIn
      consensusMarkGte: $labelConsensusMarkGte
      consensusMarkLte: $labelConsensusMarkLte
      honeypotMarkGte: $labelHoneypotMarkGte
      honeypotMarkLte: $labelHoneypotMarkLte
      createdAtGte: $labelCreatedAtGte
      createdAtLte: $labelCreatedAtLte
      skipped: $labelSkipped
    }})
}}
'''



from ...helpers import format_result
from .queries import (GQL_CREATE_ORGANIZATION, GQL_DELETE_ORGANIZATION,
                      GQL_UPDATE_ORGANIZATION)


def create_organization(client, name: str, address: str, zip_code: str, city: str, country: str):
    variables = {
        'name': name,
        'address': address,
        'zipCode': zip_code,
        'city': city,
        'country': country
    }
    result = client.execute(GQL_CREATE_ORGANIZATION, variables)
    return format_result('data', result)


def update_organization(client, organization_id: str, name: str, address: str, zip_code: str, city: str, country: str):
    variables = {
        'organizationID': organization_id,
        'name': name,
        'address': address,
        'zipCode': zip_code,
        'city': city,
        'country': country
    }
    result = client.execute(GQL_UPDATE_ORGANIZATION, variables)
    return format_result('data', result)


def delete_organization(client, organization_id: str):
    variables = {'organizationID': organization_id}
    result = client.execute(GQL_DELETE_ORGANIZATION, variables)
    return format_result('data', result)



from ...helpers import format_result
from .queries import GQL_GET_USER


def get_user(client, email: str):
    variables = {'email': email}
    result = client.execute(GQL_GET_USER, variables)
    return format_result('data', result)



from .fragments import PROJECT_FRAGMENT

GQL_GET_PROJECTS = f'''
query($userID: ID!, $searchQuery: String, $skip: Int!, $first: Int!) {{
  data: getProjects(userID: $userID, searchQuery: $searchQuery, skip: $skip, first: $first) {{
    {PROJECT_FRAGMENT}
  }}
}}
'''

GQL_GET_PROJECT = f'''
query($projectID: ID!) {{
  data: getProject(projectID: $projectID) {{
    {PROJECT_FRAGMENT}
  }}
}}
'''
